@@ -30,11 +30,11 @@ __license__   = "LGPL"
 
 import inspect
 import traceback
+import os
 import pyatspi
+import subprocess
 import sys
 
-from . import orca_state
-
 # Used to turn off all debugging.
 #
 LEVEL_OFF = 10000
@@ -116,6 +116,12 @@ debugFile = None
 eventDebugLevel  = LEVEL_FINEST
 eventDebugFilter = None
 
+# If True, we output debug information for the event queue.  We
+# use this in addition to log level to prevent debug logic from
+# bogging down event handling.
+#
+debugEventQueue = False
+
 # What module(s) should be traced if traceit is being used. By default
 # we'll just attend to ourself. (And by default, we will not enable
 # traceit.) Note that enabling this functionality will drag your system
@@ -161,6 +167,8 @@ TRACE_ROLES = []
 #
 TRACE_ONLY_PROCESSING_EVENTS = True
 
+objEvent = None
+
 def printException(level):
     """Prints out information regarding the current exception.
 
@@ -203,8 +211,8 @@ def println(level, text = ""):
             except TypeError:
                 text = "TypeError when trying to write text"
                 debugFile.writelines([text, "\n"])
-            except UnicodeDecodeError:
-                text = "UnicodeDecodeError when trying to write text"
+            except:
+                text = "Exception when trying to write text"
                 debugFile.writelines([text, "\n"])
         else:
             try:
@@ -212,9 +220,9 @@ def println(level, text = ""):
             except TypeError:
                 text = "TypeError when trying to write text"
                 sys.stderr.writelines([text, "\n"])
-            except UnicodeDecodeError:
-                text = "UnicodeDecodeError when trying to write text"
-                debugFile.writelines([text, "\n"])
+            except:
+                text = "Exception when trying to write text"
+                sys.stderr.writelines([text, "\n"])
 
 def printResult(level, result=None):
     """Prints the return result, along with information about the
@@ -227,26 +235,16 @@ def printResult(level, result=None):
     current, prev = stack[1], stack[2]
     frame = current[0]
 
-    # If we handled it, just note it in the log.
-    if frame.f_exc_type:
-        prefix = 'ERROR: '
-        suffix = '(%s:%s)' % (frame.f_exc_type.__name__, frame.f_exc_value)
-    else:
-        prefix = 'RESULT:'
-        suffix = ''
-
     # To better print arguments which are accessible objects
     args = inspect.getargvalues(frame)
-    for key, value in args.locals.items():
+    for key, value in list(args.locals.items()):
         args.locals[key] = str(value)
     fArgs = str.replace(inspect.formatargvalues(*args), "'", "")
 
     callString = 'CALL:   %s.%s (line %s) -> %s.%s%s' % (
         inspect.getmodulename(prev[1]), prev[3], prev[2],
         inspect.getmodulename(current[1]), current[3], fArgs)
-    string = '%s\n%s %s %s' \
-        % (callString, prefix, result, suffix)
-
+    string = '%s\n%s %s' % (callString, 'RESULT:', result)
     println(level, '%s' % string)
 
 def printObjectEvent(level, event, sourceInfo=None):
@@ -385,7 +383,6 @@ def _getFileAndModule(frame):
     return filename, module
 
 def _shouldTraceIt():
-    objEvent = orca_state.currentObjectEvent
     if not objEvent:
         return not TRACE_ONLY_PROCESSING_EVENTS
 
@@ -460,3 +457,60 @@ def traceit(frame, event, arg):
     println(LEVEL_ALL, output)
 
     return traceit
+
+def getOpenFDCount(pid):
+    procs = subprocess.check_output([ 'lsof', '-w', '-Ff', '-p', str(pid)])
+    procs = procs.decode('UTF-8').split('\n')
+    files = list(filter(lambda s: s and s[0] == 'f' and s[1:].isdigit(), procs))
+
+    return len(files)
+
+def getCmdline(pid):
+    try:
+        openFile = os.popen('cat /proc/%s/cmdline' % pid)
+        cmdline = openFile.read()
+        openFile.close()
+    except:
+        cmdline = '(Could not obtain cmdline)'
+    cmdline = cmdline.replace('\x00', ' ')
+
+    return cmdline
+
+def pidOf(procName):
+    openFile = subprocess.Popen('pgrep %s' % procName,
+                                shell=True,
+                                stdout=subprocess.PIPE).stdout
+    pids = openFile.read()
+    openFile.close()
+    return [int(p) for p in pids.split()]
+
+def examineProcesses():
+    desktop = pyatspi.Registry.getDesktop(0)
+    println(LEVEL_ALL, 'INFO: Desktop has %i apps:' % desktop.childCount)
+    for i, app in enumerate(desktop):
+        pid = app.get_process_id()
+        cmd = getCmdline(pid)
+        fds = getOpenFDCount(pid)
+        try:
+            name = app.name
+        except:
+            name = 'ERROR: Could not get name'
+        else:
+            if name == '':
+                name = 'WARNING: Possible hang'
+        println(LEVEL_ALL, '%3i. %s (pid: %s) %s file descriptors: %i' \
+                    % (i+1, name, pid, cmd, fds))
+
+    # Other 'suspect' processes which might not show up as accessible apps.
+    otherApps = ['apport']
+    for app in otherApps:
+        pids = pidOf(app)
+        if not pids:
+            println(LEVEL_ALL, 'INFO: no pid for %s' % app)
+            continue
+
+        for pid in pids:
+            cmd = getCmdline(pid)
+            fds = getOpenFDCount(pid)
+            println(LEVEL_ALL, 'INFO: %s (pid: %s) %s file descriptors: %i' \
+                        % (app, pid, cmd, fds))
