@@ -842,6 +842,9 @@ class Script(default.Script):
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
+        if self._inSayAll:
+            return
+
         if self.isStructuralNavigationCommand():
             return
 
@@ -909,11 +912,6 @@ class Script(default.Script):
             default.Script.onCaretMoved(self, event)
             return
 
-        try:
-            role = orca_state.locusOfFocus.getRole()
-        except:
-            orca.setLocusOfFocus(event, event.source)
-
         if orca_state.locusOfFocus.getRole() == pyatspi.ROLE_TABLE_CELL:
             default.Script.onCaretMoved(self, event)
             return
