@@ -27,9 +27,9 @@ __license__   = "LGPL"
 
 import pyatspi
 
+import orca.messages as messages
 import orca.settings_manager as settings_manager
 import orca.speech_generator as speech_generator
-from orca.orca_i18n import ngettext
 
 _settingsManager = settings_manager.getManager()
 
@@ -92,12 +92,7 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         acss = self.voice(speech_generator.SYSTEM)
         if obj and obj.getState().contains(pyatspi.STATE_EXPANDED) \
            and obj.getRole() == pyatspi.ROLE_LABEL and obj.childCount:
-            children = obj.childCount
-            # Translators: this is the number of items in a layered
-            # pane or table.
-            #
-            items = ngettext("%d item", "%d items", children) % children
-            result.append(items)
+            result.append(messages.itemCount(obj.childCount))
             result.extend(acss)
         else:
             result.extend(speech_generator.SpeechGenerator.\
@@ -154,6 +149,13 @@ class SpeechGenerator(speech_generator.SpeechGenerator):
         
     def generateSpeech(self, obj, **args):
         result = []
+        if obj.getRole() == pyatspi.ROLE_CHECK_BOX \
+           and obj.parent.getRole() == pyatspi.ROLE_MENU:
+            oldRole = self._overrideRole(pyatspi.ROLE_CHECK_MENU_ITEM, args)
+            result.extend(speech_generator.SpeechGenerator.\
+                                           generateSpeech(self, obj, **args))
+            self._restoreRole(oldRole, args)
+
         if args.get('formatType', 'unfocused') == 'basicWhereAmI' \
            and obj.getRole() == pyatspi.ROLE_TEXT:
             spinbox = self._script.utilities.ancestorWithRole(
