@@ -151,8 +151,7 @@ class Script(default.Script):
         """Returns a list of the structural navigation object types
         enabled in this script."""
 
-        enabledTypes = [StructuralNavigation.ANCHOR,
-                        StructuralNavigation.BLOCKQUOTE,
+        enabledTypes = [StructuralNavigation.BLOCKQUOTE,
                         StructuralNavigation.BUTTON,
                         StructuralNavigation.CHECK_BOX,
                         StructuralNavigation.CHUNK,
@@ -187,6 +186,9 @@ class Script(default.Script):
         - event: the Event
         """
 
+        if self._inSayAll:
+            return
+
         self._lastCaretContext = event.source, event.detail1
 
         lastKey, mods = self.utilities.lastKeyAndModifiers()
@@ -243,6 +245,9 @@ class Script(default.Script):
     def onFocusedChanged(self, event):
         """Callback for object:state-changed:focused accessibility events."""
 
+        if self._inSayAll:
+            return
+
         obj = event.source
         role = obj.getRole()
         self._lastCaretContext = obj, -1
@@ -513,16 +518,17 @@ class Script(default.Script):
 
         return True
 
-    def sayAll(self, inputEvent):
+    def sayAll(self, inputEvent, obj=None, offset=None):
         """Speaks the contents of the document beginning with the present
         location.  Overridden in this script because the sayAll could have
         been started on an object without text (such as an image).
         """
 
-        if not self.utilities.isWebKitGtk(orca_state.locusOfFocus):
-            return default.Script.sayAll(self, inputEvent)
+        obj = obj or orca_state.locusOfFocus
+        if not self.utilities.isWebKitGtk(obj):
+            return default.Script.sayAll(self, inputEvent, obj, offset)
 
-        speech.sayAll(self.textLines(orca_state.locusOfFocus),
+        speech.sayAll(self.textLines(obj, offset),
                       self.__sayAllProgressCallback)
 
         return True
@@ -548,7 +554,7 @@ class Script(default.Script):
             string, start, end = text.getTextAtOffset(offset, boundary)
         return segments
 
-    def textLines(self, obj):
+    def textLines(self, obj, offset=None):
         """Creates a generator that can be used to iterate over each line
         of a text object, starting at the caret offset.
 
@@ -560,6 +566,7 @@ class Script(default.Script):
         spoken and acss is an ACSS instance for speaking the text.
         """
 
+        self._inSayAll = False
         if not obj:
             return
 
@@ -583,6 +590,7 @@ class Script(default.Script):
         if sayAllStyle == settings.SAYALL_STYLE_SENTENCE:
             boundary = pyatspi.TEXT_BOUNDARY_SENTENCE_START
 
+        self._inSayAll = True
         offset = textObjs[0].queryText().caretOffset
         for textObj in textObjs:
             textSegments = self.getTextSegments(textObj, boundary, offset)
@@ -592,11 +600,15 @@ class Script(default.Script):
                 textSegments.append([roleName, 0, -1, voice])
 
             for (string, start, end, voice) in textSegments:
-                yield [speechserver.SayAllContext(textObj, string, start, end),
-                       voice]
+                context = speechserver.SayAllContext(textObj, string, start, end)
+                self._sayAllContexts.append(context)
+                yield [context, voice]
 
             offset = 0
 
+        self._inSayAll = False
+        self._sayAllContexts = []
+
     def __sayAllProgressCallback(self, context, progressType):
         if progressType == speechserver.SayAllContext.PROGRESS:
             return
@@ -608,6 +620,15 @@ class Script(default.Script):
         text = obj.queryText()
 
         if progressType == speechserver.SayAllContext.INTERRUPTED:
+            if isinstance(orca_state.lastInputEvent, input_event.KeyboardEvent):
+                lastKey = orca_state.lastInputEvent.event_string
+                if lastKey == "Down" and self._fastForwardSayAll(context):
+                    return
+                elif lastKey == "Up" and self._rewindSayAll(context):
+                    return
+
+            self._inSayAll = False
+            self._sayAllContexts = []
             text.setCaretOffset(offset)
             return
 
