@@ -71,13 +71,6 @@ class SpeechServer(speechserver.SpeechServer):
     
     DEFAULT_SERVER_ID = 'default'
     _SERVER_NAMES = {DEFAULT_SERVER_ID: guilabels.DEFAULT_SYNTHESIZER}
-    
-    KEY_NAMES = {
-        '_':     'underscore',
-        ' ':     'space',
-        'space': 'space',
-        '"':     'double-quote',
-        }
 
     def getFactoryName():
         return guilabels.SPEECH_DISPATCHER
@@ -195,6 +188,8 @@ class SpeechServer(speechserver.SpeechServer):
                               "Trying to reconnect.")
                 self.reset()
                 return command(*args, **kwargs)
+            except:
+                pass
         else:
             # It is not possible tho catch the error with older SD versions. 
             return command(*args, **kwargs)
@@ -243,7 +238,7 @@ class SpeechServer(speechserver.SpeechServer):
             elif acss_property == ACSS.AVERAGE_PITCH:
                 method(5.0)
                 current[acss_property] = 5.0
-            elif acss_property == ACSS.FAMILY and current.get(acss_property) \
+            elif acss_property == ACSS.FAMILY \
                     and acss == settings.voices[settings.DEFAULT_VOICE]:
                 # We need to explicitly reset (at least) the family.
                 # See bgo#626072.
@@ -292,7 +287,7 @@ class SpeechServer(speechserver.SpeechServer):
         if isinstance(text, ACSS):
             text = ''
         text = self.__addVerbalizedPunctuation(text)
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        if orca_state.activeScript:
             text = orca_state.activeScript.\
                 utilities.adjustForPronunciation(text)
 
@@ -342,9 +337,9 @@ class SpeechServer(speechserver.SpeechServer):
     def _cancel(self):
         self._send_command(self._client.cancel)
 
-    def _change_default_speech_rate(self, decrease=False):
+    def _change_default_speech_rate(self, step, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
-        delta = settings.speechRateDelta * (decrease and -1 or +1)
+        delta = step * (decrease and -1 or +1)
         try:
             rate = acss[ACSS.RATE]
         except KeyError:
@@ -356,9 +351,9 @@ class SpeechServer(speechserver.SpeechServer):
         self.speak(decrease and messages.SPEECH_SLOWER \
                    or messages.SPEECH_FASTER, acss=acss)
 
-    def _change_default_speech_pitch(self, decrease=False):
+    def _change_default_speech_pitch(self, step, decrease=False):
         acss = settings.voices[settings.DEFAULT_VOICE]
-        delta = settings.speechPitchDelta * (decrease and -1 or +1)
+        delta = step * (decrease and -1 or +1)
         try:
             pitch = acss[ACSS.AVERAGE_PITCH]
         except KeyError:
@@ -390,7 +385,10 @@ class SpeechServer(speechserver.SpeechServer):
         except AttributeError:
             pass
         else:
-            voices += self._send_command(list_synthesis_voices)
+            try:
+                voices += self._send_command(list_synthesis_voices)
+            except:
+                pass
         families = [speechserver.VoiceFamily({ \
               speechserver.VoiceFamily.NAME: name,
               #speechserver.VoiceFamily.GENDER: speechserver.VoiceFamily.MALE,
@@ -406,10 +404,6 @@ class SpeechServer(speechserver.SpeechServer):
         if text:
             self._speak(text, acss)
 
-    def queueText(self, text="", acss=None):
-        if text:
-            self._speak(text, acss)
-
     def speakUtterances(self, utteranceList, acss=None, interrupt=True):
         #if interrupt:
         #    self._cancel()
@@ -431,37 +425,37 @@ class SpeechServer(speechserver.SpeechServer):
             self._send_command(self._client.char, character)
             return
 
-        if orca_state.activeScript and orca_state.usePronunciationDictionary:
+        if orca_state.activeScript:
             name = orca_state.activeScript.\
                 utilities.adjustForPronunciation(name)
         self.speak(name, acss)
 
     def speakKeyEvent(self, event):
-        if event.isPrintableKey():
-            # We currently only handle printable characters by Speech
-            # Dispatcher's KEY command.  For other keys, such as Ctrl, Shift
-            # etc. we prefer Orca's verbalization.
-            if event.event_string.isupper():
-                acss = settings.voices[settings.UPPERCASE_VOICE]
-            else:
-                acss = None
-            key = self.KEY_NAMES.get(event.event_string, event.event_string)
-            self._apply_acss(acss)
-            self._send_command(self._client.key, key)
+        if event.isPrintableKey() and event.event_string.isupper():
+            acss = settings.voices[settings.UPPERCASE_VOICE]
         else:
-            return super(SpeechServer, self).speakKeyEvent(event)
+            acss = ACSS(settings.voices[settings.DEFAULT_VOICE])
+
+        event_string = event.getKeyName()
+        if orca_state.activeScript:
+            event_string = orca_state.activeScript.\
+                utilities.adjustForPronunciation(event_string)
+
+        lockingStateString = event.getLockingStateString()
+        event_string = "%s %s" % (event_string, lockingStateString)
+        self.speak(event_string, acss=acss)
 
     def increaseSpeechRate(self, step=5):
-        self._change_default_speech_rate()
+        self._change_default_speech_rate(step)
 
     def decreaseSpeechRate(self, step=5):
-        self._change_default_speech_rate(decrease=True)
+        self._change_default_speech_rate(step, decrease=True)
 
     def increaseSpeechPitch(self, step=0.5):
-        self._change_default_speech_pitch()
+        self._change_default_speech_pitch(step)
 
     def decreaseSpeechPitch(self, step=0.5):
-        self._change_default_speech_pitch(decrease=True)
+        self._change_default_speech_pitch(step, decrease=True)
 
     def stop(self):
         self._cancel()
